package client.world;

import java.util.List;
import java.util.function.Predicate;

import common.biome.Biome;
import common.block.Block;
import common.entity.Entity;
import common.init.Blocks;
import common.tileentity.TileEntity;
import common.util.BlockPos;
import common.util.BoundingBox;
import common.world.LightType;
import common.world.State;

public class ChunkEmpty extends ChunkClient {
	private final int liquidY;
	private final State liquid;
	private final Block liquidBlock;
	
	public ChunkEmpty(WorldClient world) {
		super(world, 0, 0);
		this.liquidY = world.dimension.getSeaLevel() - 1;
		this.liquid = world.dimension.getLiquid();
		this.liquidBlock = this.liquid.getBlock();
	}

	public int getHeight(int x, int z) {
		return this.liquidY;
	}

	public Block getBlock(BlockPos pos) {
		return pos.getY() < this.liquidY ? Blocks.bedrock : (pos.getY() == this.liquidY ? this.liquidBlock : Blocks.air);
	}

	public int getLight(LightType type, BlockPos pos) {
		return type.defValue;
	}

	public void setLight(LightType type, BlockPos pos, int value) {
	}

	public int getLightSub(BlockPos pos, int amount) {
		return 0;
	}

	public void addEntity(Entity entity) {
	}

	public void removeEntity(Entity entity) {
	}

	public boolean canSeeSky(BlockPos pos) {
		return pos.getY() > this.liquidY;
	}

	public TileEntity getTileEntity(BlockPos pos, TileEntity.EnumCreateEntityType type) {
		return null;
	}

	public void addTileEntity(BlockPos pos, TileEntity tile) {
	}

	public void removeTileEntity(BlockPos pos) {
	}
	
	public void getEntities(Entity exclude, BoundingBox bb, List<Entity> list, Predicate<? super Entity> pred) {
	}

	public <T extends Entity> void getEntities(Class<? extends T> clazz, BoundingBox bb, List<T> list, Predicate<? super T> pred) {
	}

	public boolean isDummy() {
		return true;
	}

	public boolean isEmpty(int bottom, int top) {
		return top < 0 || bottom > this.liquidY;
	}

	public State getState(BlockPos pos) {
		return pos.getY() < this.liquidY ? Blocks.bedrock.getState() : (pos.getY() == this.liquidY ? this.liquid : Blocks.air.getState());
	}

	public State setState(BlockPos pos, State state) {
		return null;
	}
	
	public BlockPos getPrecipitation(BlockPos pos) {
		return new BlockPos(pos.getX(), this.liquidY + 1, pos.getZ());
	}

	public void update(boolean noGaps) {
	}
	
	public void onChunkUnload() {
	}

	public boolean isPopulated() {
		return true;
	}

	public void setData(byte[] data, int[] extend, boolean biomes) {
	}

	public Biome getBiome(BlockPos pos) {
		return Biome.DEF_BIOME;
	}

	public void setBiome(BlockPos pos, Biome biome) {
	}

	public void resetRelight() {
	}

	public void enqueueRelight() {
	}

	public void checkLight() {
	}

	public boolean isLoaded() {
		return false;
	}

	public void setLoaded() {
	}

	public int getLowest() {
		return 0;
	}
}
