package common.block;

import java.util.List;

import common.color.DyeColor;
import common.item.CheatTab;
import common.item.Item;
import common.item.ItemStack;
import common.model.Model;
import common.model.ModelProvider;
import common.properties.IProperty;
import common.properties.PropertyEnum;
import common.world.State;

public class BlockColored extends Block {
	public static final PropertyEnum<DyeColor> COLOR = PropertyEnum.<DyeColor>create("color", DyeColor.class);

	public BlockColored(Material material) {
		super(material);
		this.setDefaultState(this.getBaseState().withProperty(COLOR, DyeColor.WHITE));
		this.setTab(CheatTab.BLOCKS);
	}

	public int damageDropped(State state) {
		return state.getValue(COLOR).getMetadata();
	}

	public void getSubBlocks(Item item, CheatTab tab, List<ItemStack> list) {
		for(DyeColor color : DyeColor.values()) {
			list.add(new ItemStack(item, 1, color.getMetadata()));
		}
	}

	public State getStateFromMeta(int meta) {
		return this.getState().withProperty(COLOR, DyeColor.byMetadata(meta));
	}

	public int getMetaFromState(State state) {
		return state.getValue(COLOR).getMetadata();
	}

	protected IProperty[] getProperties() {
		return new IProperty[] {COLOR};
	}

	public Model getModel(ModelProvider provider, String name, State state) {
		return provider.getModel(state.getValue(COLOR).getName() + "_" + name).add().all();
	}
}
