package common.init;

import java.util.Map;

import common.block.Block;
import common.collect.Maps;

public abstract class FlammabilityRegistry {
	private static final Map<Block, Integer> ENCOURAGEMENT = Maps.<Block, Integer>newIdentityHashMap();
	private static final Map<Block, Integer> FLAMMABILITY = Maps.<Block, Integer>newIdentityHashMap();

	private static void setFlammable(Block block, int encouragement, int flammability) {
		ENCOURAGEMENT.put(block, encouragement);
		FLAMMABILITY.put(block, flammability);
	}

	static void register() {
		for(WoodType wood : WoodType.values()) {
			setFlammable(BlockRegistry.getRegisteredBlock(wood.getName() + "_planks"), 5, 20);
			setFlammable(BlockRegistry.getRegisteredBlock(wood.getName() + "_slab"), 5, 20);
			setFlammable(BlockRegistry.getRegisteredBlock(wood.getName() + "_stairs"), 5, 20);
			setFlammable(BlockRegistry.getRegisteredBlock(wood.getName() + "_fence"), 5, 20);
			setFlammable(BlockRegistry.getRegisteredBlock(wood.getName() + "_fence_gate"), 5, 20);
			setFlammable(BlockRegistry.getRegisteredBlock(wood.getName() + "_log"), 5, 5);
			setFlammable(BlockRegistry.getRegisteredBlock(wood.getName() + "_leaves"), 30, 60);
			setFlammable(BlockRegistry.getRegisteredBlock(wood.getName() + "_sapling"), 15, 100);
		}
		setFlammable(Blocks.bookshelf, 30, 20);
		setFlammable(Blocks.tnt, 15, 100);
		setFlammable(Blocks.tallgrass, 60, 100);
		setFlammable(Blocks.double_plant, 60, 100);
		setFlammable(Blocks.flower, 60, 100);
		setFlammable(Blocks.deadbush, 60, 100);
		setFlammable(Blocks.dry_leaves, 60, 100);
		setFlammable(Blocks.wool, 30, 60);
		setFlammable(Blocks.vine, 15, 100);
		setFlammable(Blocks.coal_block, 5, 5);
		setFlammable(Blocks.hay_block, 60, 20);
		setFlammable(Blocks.carpet, 60, 20);
	}

	public static int getFlammability(Block block) {
		Integer value = FLAMMABILITY.get(block);
		return value == null ? 0 : value.intValue();
	}

	public static int getEncouragement(Block block) {
		Integer value = ENCOURAGEMENT.get(block);
		return value == null ? 0 : value.intValue();
	}
}
