package common.item;

import java.util.Map;
import java.util.Set;

import common.attributes.Attribute;
import common.attributes.AttributeModifier;
import common.attributes.Attributes;
import common.block.Block;
import common.collect.Sets;
import common.entity.types.EntityLiving;
import common.init.ToolMaterial;
import common.model.Transforms;
import common.util.BlockPos;
import common.world.World;

public abstract class ItemTool extends Item {
	private final float efficiency;
	private final int damage;
	private final ToolMaterial material;

	public ItemTool(int damage, ToolMaterial material) {
		this.material = material;
		this.maxStackSize = 1;
		this.setMaxDamage(material.getDurability());
		this.efficiency = material.getEfficiency();
		this.damage = damage + material.getDamage();
		this.setTab(CheatTab.TOOLS);
	}

	public abstract boolean canUseOn(ItemStack stack, Block block);

	public float getStrVsBlock(ItemStack stack, Block block) {
		return !this.canUseOn(stack, block) ? 1.0F : this.efficiency;
	}

	public boolean hitEntity(ItemStack stack, EntityLiving target, EntityLiving attacker) {
		stack.damageItem(2, attacker);
		return true;
	}

	public boolean onBlockDestroyed(ItemStack stack, World world, Block block, BlockPos pos, EntityLiving player) {
		if(block.getBlockHardness(world, pos) != 0.0f)
			stack.damageItem(1, player);
		return true;
	}

	public ToolMaterial getToolMaterial() {
		return this.material;
	}

	public int getItemEnchantability() {
		return this.material.getEnchantability();
	}

	public boolean getIsRepairable(ItemStack stack, ItemStack repair) {
		return this.material.isRepairItem(repair.getItem()) ? true : super.getIsRepairable(stack, repair);
	}

	public Map<Attribute, Set<AttributeModifier>> getItemAttributeModifiers() {
		Map<Attribute, Set<AttributeModifier>> mods = super.getItemAttributeModifiers();
		mods.put(Attributes.ATTACK_DAMAGE,
				Sets.newHashSet(new AttributeModifier(Attributes.ITEM_VAL_ID, "Tool modifier", this.damage, false)));
		return mods;
	}

	public boolean isMagnetic() {
		return this.material.isMagnetic();
	}

	public Transforms getTransform() {
		return Transforms.TOOL;
	}

	public boolean canBeWielded() {
		return true;
	}
}
