package common.packet;

import java.io.IOException;

import common.network.IClientPlayer;
import common.network.Packet;
import common.network.PacketBuffer;

public class SPacketConfirmTransaction implements Packet<IClientPlayer>
{
    private int windowId;
    private short actionNumber;
    private boolean matching;

    public SPacketConfirmTransaction()
    {
    }

    public SPacketConfirmTransaction(int windowIdIn, short actionNumberIn, boolean matching)
    {
        this.windowId = windowIdIn;
        this.actionNumber = actionNumberIn;
        this.matching = matching;
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(IClientPlayer handler)
    {
        handler.handleConfirmTransaction(this);
    }

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.windowId = buf.readUnsignedByte();
        this.actionNumber = buf.readShort();
        this.matching = buf.readBoolean();
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeByte(this.windowId);
        buf.writeShort(this.actionNumber);
        buf.writeBoolean(this.matching);
    }

    public int getWindowId()
    {
        return this.windowId;
    }

    public short getActionNumber()
    {
        return this.actionNumber;
    }

    public boolean isMatching()
    {
        return this.matching;
    }
}
