package common.packet;

import java.io.IOException;

import common.item.ItemStack;
import common.network.IClientPlayer;
import common.network.Packet;
import common.network.PacketBuffer;

public class SPacketEntityEquipment implements Packet<IClientPlayer>
{
    private int entityID;
    private int equipmentSlot;
    private ItemStack itemStack;

    public SPacketEntityEquipment()
    {
    }

    public SPacketEntityEquipment(int entityIDIn, int p_i45221_2_, ItemStack itemStackIn)
    {
        this.entityID = entityIDIn;
        this.equipmentSlot = p_i45221_2_;
        this.itemStack = itemStackIn == null ? null : itemStackIn.copy();
    }

    /**
     * Reads the raw packet data from the data stream.
     */
    public void readPacketData(PacketBuffer buf) throws IOException
    {
        this.entityID = buf.readVarInt();
        this.equipmentSlot = buf.readShort();
        this.itemStack = buf.readItemStack();
    }

    /**
     * Writes the raw packet data to the data stream.
     */
    public void writePacketData(PacketBuffer buf) throws IOException
    {
        buf.writeVarInt(this.entityID);
        buf.writeShort(this.equipmentSlot);
        buf.writeItemStack(this.itemStack);
    }

    /**
     * Passes this Packet on to the NetHandler for processing.
     */
    public void processPacket(IClientPlayer handler)
    {
        handler.handleEntityEquipment(this);
    }

    public ItemStack getItemStack()
    {
        return this.itemStack;
    }

    public int getEntityID()
    {
        return this.entityID;
    }

    public int getEquipmentSlot()
    {
        return this.equipmentSlot;
    }
}
