package server.biome;

import common.biome.Biome;
import common.block.BlockDirectional;
import common.block.foliage.BlockFlower;
import common.entity.npc.EntitySlime;
import common.init.Blocks;
import common.rng.Random;
import common.util.BlockPos;
import common.util.Facing;
import server.world.WorldServer;
import server.worldgen.ChunkPrimer;
import server.worldgen.tree.WorldGenTree;

public class BiomeSwamp extends GenBiome
{
    protected BiomeSwamp()
    {
        super(Biome.SWAMPLAND);
        this.treesPerChunk = 2;
        this.flowersPerChunk = 1;
        this.deadBushPerChunk = 1;
        this.mushroomsPerChunk = 8;
        this.reedsPerChunk = 10;
        this.clayPerChunk = 1;
        this.waterlilyPerChunk = 4;
        this.sandPerChunk2 = 0;
        this.sandPerChunk = 0;
        this.grassPerChunk = 5;
        this.mobs.add(new RngSpawn(EntitySlime.class, 1, 1, 1));
		this.disableBeach();
    }

    public WorldGenTree genBigTreeChance(Random rand)
    {
        return this.worldGeneratorSwamp;
    }

    public BlockFlower.EnumFlowerType pickRandomFlower(Random rand, BlockPos pos)
    {
        return BlockFlower.EnumFlowerType.BLUE_ORCHID;
    }

    public void genTerrainBlocks(WorldServer worldIn, Random rand, ChunkPrimer chunkPrimerIn, int x, int z, double noiseVal)
    {
        double d0 = GRASS_NOISE.generate((double)x * 0.25D, (double)z * 0.25D);

        if (d0 > 0.0D)
        {
            int i = x & 15;
            int j = z & 15;

            for (int k = chunkPrimerIn.height - 1; k >= 0; --k)
            {
                if (chunkPrimerIn.get(j, k, i).getBlock() != Blocks.air)
                {
                    if (k == 62 && chunkPrimerIn.get(j, k, i).getBlock() != Blocks.water)
                    {
                        chunkPrimerIn.set(j, k, i, Blocks.water.getState());

                        if (d0 < 0.12D)
                        {
                            chunkPrimerIn.set(j, k + 1, i, Blocks.waterlily.getState().withProperty(BlockDirectional.FACING, Facing.randHorizontal(rand)));
                        }
                    }

                    break;
                }
            }
        }

        this.generateBiomeTerrain(worldIn, rand, chunkPrimerIn, x, z, noiseVal);
    }
}
