package server.biome;

import common.biome.Biome;
import common.block.foliage.BlockFlower;
import common.entity.animal.EntityBat;
import common.entity.animal.EntityMouse;
import common.entity.animal.EntityRabbit;
import common.entity.npc.EntityCultivator;
import common.init.Blocks;
import common.rng.Random;
import common.rng.WeightedList;
import common.util.BlockPos;
import server.world.WorldServer;
import server.worldgen.FeatureGenerator;
import server.worldgen.feature.WorldGenSpikes;
import server.worldgen.foliage.WorldGenMushroom;
import server.worldgen.tree.WorldGenBaseTree;
import server.worldgen.tree.WorldGenBigTree;
import server.worldgen.tree.WorldGenTree;

public class BiomeTian extends GenBiome
{
    protected FeatureGenerator spikeGen = new WorldGenSpikes(Blocks.tian_soil, 128, 2, 3, Blocks.obsidian.getState(), true);
    protected FeatureGenerator mushroomBlueGen = new WorldGenMushroom(Blocks.blue_mushroom);
    protected WorldGenTree treeGen1 = new WorldGenBigTree(false, Blocks.tian_log.getState(), Blocks.tian_leaves.getState())
        	.setHeightLimit(6, 8);
    protected WorldGenTree treeGen2 = new WorldGenBigTree(false, Blocks.tian_log.getState(), Blocks.tian_leaves.getState())
        	.setHeightLimit(14, 20);
    protected WorldGenTree treeGen3 = new WorldGenBaseTree(false, Blocks.tian_log.getState(), Blocks.tian_leaves.getState());
    protected WorldGenTree treeGen4 = new WorldGenBigTree(false, Blocks.tian_log.getState(), Blocks.tian_leaves.getState())
        	.setHeightLimit(12, 15);
    
    public BiomeTian()
    {
        super(Biome.TIAN);
        this.topBlock = Blocks.tian_soil.getState();
        this.fillerBlock = Blocks.tian.getState();
        this.mushroomsPerChunk = -1;
        this.grassPerChunk = 0;
        this.treesPerChunk = 1;
    }
    
    protected void addMobs(WeightedList<RngSpawn> mobs) {
//        mobs.add(new Biome.RngSpawn(EntityHaunter.class, 50, 4, 4));
        mobs.add(new RngSpawn(EntityCultivator.class, 50, 1, 1));
        mobs.add(new RngSpawn(EntityRabbit.class, 10, 3, 10));
        mobs.add(new RngSpawn(EntityBat.class, 10, 8, 8));
        mobs.add(new RngSpawn(EntityMouse.class, 10, 8, 8));
    }

    public BlockFlower.EnumFlowerType pickRandomFlower(Random rand, BlockPos pos)
    {
        return BlockFlower.EnumFlowerType.BLACK_LOTUS;
    }
    
    public WorldGenTree genBigTreeChance(Random rand)
    {
    	return rand.pick(rand.chance(this.treeGen2, this.treeGen1, 4), rand.chance(this.treeGen3, this.treeGen4, 15));
    }

    public void decorate(WorldServer worldIn, Random rand, BlockPos pos)
    {
        super.decorate(worldIn, rand, pos);

        for (int l3 = 0; l3 < 2; ++l3)
        {
            if (rand.chance(4))
            {
                int i8 = rand.chOffset();
                int l11 = rand.chOffset();
                BlockPos blockpos2 = worldIn.getHeight(pos.add(i8, 0, l11));
                this.mushroomBlueGen.generate(worldIn, rand, blockpos2);
            }
        }

        if (rand.chance(4))
        {
            int i = rand.chOffset();
            int j = rand.chOffset();
            this.spikeGen.generate(worldIn, rand, worldIn.getTopSolidOrLiquidBlock(pos.add(i, 0, j)));
        }
    }
}
