package server.worldgen.foliage;

import common.block.Block;
import common.block.Material;
import common.block.foliage.BlockLeaves;
import common.init.Blocks;
import common.rng.Random;
import common.util.BlockPos;
import common.world.State;
import server.world.WorldServer;
import server.worldgen.tree.WorldGenBaseTree;

public class WorldGenShrub extends WorldGenBaseTree
{
    private final State leavesMetadata;
    private final State woodMetadata;

    public WorldGenShrub(State log, State leaves)
    {
        super(false);
        this.woodMetadata = log;
        this.leavesMetadata = leaves.withProperty(BlockLeaves.DECAY, Boolean.valueOf(false));
    }

    public boolean generate(WorldServer worldIn, Random rand, BlockPos position)
    {
        Block block;

        while (((block = worldIn.getState(position).getBlock()) == Blocks.air || block.getMaterial() == Material.LEAVES) && position.getY() > 0)
        {
            position = position.down();
        }

        Block block1 = worldIn.getState(position).getBlock();

        if (block1 == Blocks.dirt || block1 == Blocks.grass)
        {
            position = position.up();
            this.setBlockAndNotifyAdequately(worldIn, position, this.woodMetadata);

            for (int i = position.getY(); i <= position.getY() + 2; ++i)
            {
                int j = i - position.getY();
                int k = 2 - j;

                for (int l = position.getX() - k; l <= position.getX() + k; ++l)
                {
                    int i1 = l - position.getX();

                    for (int j1 = position.getZ() - k; j1 <= position.getZ() + k; ++j1)
                    {
                        int k1 = j1 - position.getZ();

                        if (Math.abs(i1) != k || Math.abs(k1) != k || rand.zrange(2) != 0)
                        {
                            BlockPos blockpos = new BlockPos(l, i, j1);

                            if (!worldIn.getState(blockpos).getBlock().isFullBlock())
                            {
                                this.setBlockAndNotifyAdequately(worldIn, blockpos, this.leavesMetadata.withProperty(BlockLeaves.TYPE, worldIn.getLeavesGen(blockpos)));
                            }
                        }
                    }
                }
            }
        }

        return true;
    }
}
