package server.worldgen.tree;

import common.block.Block;
import common.block.Material;
import common.block.foliage.BlockLeaves;
import common.block.foliage.BlockVine;
import common.init.Blocks;
import common.properties.PropertyBool;
import common.rng.Random;
import common.util.BlockPos;
import common.world.State;
import server.world.WorldServer;

public class WorldGenSwamp extends WorldGenTree
{
    private static final State field_181648_a = Blocks.oak_log.getState();
//    		.withProperty(BlockOldLog.VARIANT, BlockPlanks.EnumType.OAK);
    private static final State field_181649_b = Blocks.oak_leaves.getState()
//    		.withProperty(BlockOldLeaf.VARIANT, BlockPlanks.EnumType.OAK)
    		.withProperty(BlockLeaves.DECAY, Boolean.valueOf(false));

    public WorldGenSwamp()
    {
        super(false);
    }

    public boolean generate(WorldServer worldIn, Random rand, BlockPos position)
    {
        int i;

        for (i = rand.zrange(4) + 5; worldIn.getState(position.down()).getBlock().getMaterial() == Material.WATER; position = position.down())
        {
            ;
        }

        boolean flag = true;

        if (position.getY() >= 1 && position.getY() + i + 1 <= 512)
        {
            for (int j = position.getY(); j <= position.getY() + 1 + i; ++j)
            {
                int k = 1;

                if (j == position.getY())
                {
                    k = 0;
                }

                if (j >= position.getY() + 1 + i - 2)
                {
                    k = 3;
                }

                BlockPos.MutableBlockPos blockpos$mutableblockpos = new BlockPos.MutableBlockPos();

                for (int l = position.getX() - k; l <= position.getX() + k && flag; ++l)
                {
                    for (int i1 = position.getZ() - k; i1 <= position.getZ() + k && flag; ++i1)
                    {
                        if (j >= 0 && j < 512)
                        {
                            Block block = worldIn.getState(blockpos$mutableblockpos.set(l, j, i1)).getBlock();

                            if (block != Blocks.air && block.getMaterial() != Material.LEAVES)
                            {
                                if (block != Blocks.water && block != Blocks.flowing_water)
                                {
                                    flag = false;
                                }
                                else if (j > position.getY())
                                {
                                    flag = false;
                                }
                            }
                        }
                        else
                        {
                            flag = false;
                        }
                    }
                }
            }

            if (!flag)
            {
                return false;
            }
            else
            {
                Block block1 = worldIn.getState(position.down()).getBlock();

                if ((block1 == Blocks.grass || block1 == Blocks.dirt) && position.getY() < 512 - i - 1)
                {
                    this.setBaseBlock(worldIn, position.down());

                    for (int l1 = position.getY() - 3 + i; l1 <= position.getY() + i; ++l1)
                    {
                        int k2 = l1 - (position.getY() + i);
                        int i3 = 2 - k2 / 2;

                        for (int k3 = position.getX() - i3; k3 <= position.getX() + i3; ++k3)
                        {
                            int l3 = k3 - position.getX();

                            for (int j1 = position.getZ() - i3; j1 <= position.getZ() + i3; ++j1)
                            {
                                int k1 = j1 - position.getZ();

                                if (Math.abs(l3) != i3 || Math.abs(k1) != i3 || rand.zrange(2) != 0 && k2 != 0)
                                {
                                    BlockPos blockpos = new BlockPos(k3, l1, j1);

                                    if (!worldIn.getState(blockpos).getBlock().isFullBlock())
                                    {
                                        this.setBlockAndNotifyAdequately(worldIn, blockpos, field_181649_b.withProperty(BlockLeaves.TYPE, worldIn.getLeavesGen(blockpos)));
                                    }
                                }
                            }
                        }
                    }

                    for (int i2 = 0; i2 < i; ++i2)
                    {
                        Block block2 = worldIn.getState(position.up(i2)).getBlock();

                        if (block2 == Blocks.air || block2.getMaterial() == Material.LEAVES || block2 == Blocks.flowing_water || block2 == Blocks.water)
                        {
                            this.setBlockAndNotifyAdequately(worldIn, position.up(i2), field_181648_a);
                        }
                    }

                    for (int j2 = position.getY() - 3 + i; j2 <= position.getY() + i; ++j2)
                    {
                        int l2 = j2 - (position.getY() + i);
                        int j3 = 2 - l2 / 2;
                        BlockPos.MutableBlockPos blockpos$mutableblockpos1 = new BlockPos.MutableBlockPos();

                        for (int i4 = position.getX() - j3; i4 <= position.getX() + j3; ++i4)
                        {
                            for (int j4 = position.getZ() - j3; j4 <= position.getZ() + j3; ++j4)
                            {
                                blockpos$mutableblockpos1.set(i4, j2, j4);

                                if (worldIn.getState(blockpos$mutableblockpos1).getBlock().getMaterial() == Material.LEAVES)
                                {
                                    BlockPos blockpos3 = blockpos$mutableblockpos1.west();
                                    BlockPos blockpos4 = blockpos$mutableblockpos1.east();
                                    BlockPos blockpos1 = blockpos$mutableblockpos1.north();
                                    BlockPos blockpos2 = blockpos$mutableblockpos1.south();

                                    if (rand.zrange(4) == 0 && worldIn.getState(blockpos3).getBlock() == Blocks.air)
                                    {
                                        this.func_181647_a(worldIn, blockpos3, BlockVine.EAST);
                                    }

                                    if (rand.zrange(4) == 0 && worldIn.getState(blockpos4).getBlock() == Blocks.air)
                                    {
                                        this.func_181647_a(worldIn, blockpos4, BlockVine.WEST);
                                    }

                                    if (rand.zrange(4) == 0 && worldIn.getState(blockpos1).getBlock() == Blocks.air)
                                    {
                                        this.func_181647_a(worldIn, blockpos1, BlockVine.SOUTH);
                                    }

                                    if (rand.zrange(4) == 0 && worldIn.getState(blockpos2).getBlock() == Blocks.air)
                                    {
                                        this.func_181647_a(worldIn, blockpos2, BlockVine.NORTH);
                                    }
                                }
                            }
                        }
                    }

                    return true;
                }
                else
                {
                    return false;
                }
            }
        }
        else
        {
            return false;
        }
    }

    private void func_181647_a(WorldServer p_181647_1_, BlockPos p_181647_2_, PropertyBool p_181647_3_)
    {
        State iblockstate = Blocks.vine.getState().withProperty(p_181647_3_, Boolean.valueOf(true));
        this.setBlockAndNotifyAdequately(p_181647_1_, p_181647_2_, iblockstate);
        int i = 4;

        for (p_181647_2_ = p_181647_2_.down(); p_181647_1_.getState(p_181647_2_).getBlock() == Blocks.air && i > 0; --i)
        {
            this.setBlockAndNotifyAdequately(p_181647_1_, p_181647_2_, iblockstate);
            p_181647_2_ = p_181647_2_.down();
        }
    }
}
